package fr.inria.dioptase.examples;

import java.io.IOException;

import fr.inria.dioptase.Dioptase;
import fr.inria.dioptase.atomic.Processor;
import fr.inria.dioptase.atomic.Producer;
import fr.inria.dioptase.driver.discovery.BogusDiscoveryDriverLoader;
import fr.inria.dioptase.driver.network.JSENetworkDriverLoader;
import fr.inria.dioptase.driver.storage.JSEStorageDriverLoader;
import fr.inria.dioptase.driver.thing.BogusThingDriverLoader;
import fr.inria.dioptase.stream.Stream;
import fr.inria.dioptase.stream.StreamCallback;
import fr.inria.dioptase.stream.StreamItem;
import fr.inria.dioptase.stream.StreamUrl;
import fr.inria.dioptase.thing.Sensor;
import fr.inria.dioptase.utils.Logger;
import fr.inria.dioptase.utils.Logger.Level;


/**
 * @author Benjamin Billet
 *
 */
public class DynamicInterpretedTask
{
	public static void main(String[] args)
	throws IOException
	{
		Logger.setLevel(Level.INFO);
		Dioptase dioptase = Dioptase.getInstance(5000);
		
		dioptase.initDrivers(new JSENetworkDriverLoader(), new BogusThingDriverLoader(), new JSEStorageDriverLoader(), new BogusDiscoveryDriverLoader());
		dioptase.start();
			
		// get the bogus temperature sensor and instanciate a producer
		Sensor sensor = dioptase.getThingProperties().getSensor("temperatureSensor");
		Producer producer = dioptase.createSensorProducer("temperatureProducer", sensor, 1500);
		
		// deploy a processor with an interpreted task (write the value in the output if the temperature is greater than 10°) using the deployment service
		Utils.deployInterpretedTask(dioptase, "temperatureFilter", getFILTERDescription(producer.getStream(), "temperature"), producer.getStream().getSchema(), new StreamUrl("127.0.0.1", 5000, "temperatureProducer"));
		Processor processor = dioptase.findProcessor("temperatureFilter");
		
		System.out.println("============ Producer schema:");
		System.out.println(producer.getId() + " " + producer.getStream().getSchema());
		System.out.println("============ Processor schema:");
		System.out.println(processor.getId() + " " + processor.getStream().getSchema());
		
		System.out.println("============ Outputs (press enter to exit):");
		
		// read the output of the producer
		producer.getDispatcher().addCallback("tempCallback", new StreamCallback()
		{				
			@Override
			public void newItemReceived(Stream stream, StreamItem item)
			{
				System.out.println(item.getTimestamp() + ": " + stream.getName() + " produces " + item.getTuple() + ", " + stream.getSchema());
			}
		});
		
		// read the output of the processor
		processor.getDispatcher().addCallback("filterCallback", new StreamCallback()
		{				
			@Override
			public void newItemReceived(Stream stream, StreamItem item)
			{
				System.out.println(item.getTimestamp() + ": " + stream.getName() + " produces " + item.getTuple() + ", " + stream.getSchema());
			}
		});
		
		System.in.read();
		dioptase.shutdown();
	}
	
	public static String getFILTERDescription(Stream input, String attribute)
	{
		return
				"work:" +
				"	(def \"diff\" (getNewItems \"" + input.getName() + "\"))" +
				"	(if (> (size diff) 0)" +
				"		(foreach \"i\" diff" +
				"			(if (> (itemvalue i \"" + attribute + "\") 10)" +
				"				(write i)\r\n" +
				"		)\r\n" +
				"	)\r\n";
	}
}
