package fr.inria.dioptase.examples;

import java.io.IOException;

import fr.inria.dioptase.Dioptase;
import fr.inria.dioptase.atomic.Producer;
import fr.inria.dioptase.driver.discovery.BogusDiscoveryDriverLoader;
import fr.inria.dioptase.driver.network.JSENetworkDriverLoader;
import fr.inria.dioptase.driver.storage.JSEStorageDriverLoader;
import fr.inria.dioptase.driver.thing.BogusThingDriverLoader;
import fr.inria.dioptase.thing.Sensor;
import fr.inria.dioptase.utils.Logger;
import fr.inria.dioptase.utils.Logger.Level;


/**
 * @author Benjamin Billet
 *
 */
public class Mashup
{
	public static void main(String[] args)
	throws IOException
	{
		Logger.setLevel(Level.INFO);
		Dioptase dioptase = Dioptase.getInstance(5000);
		
		dioptase.initDrivers(new JSENetworkDriverLoader(), new BogusThingDriverLoader(), new JSEStorageDriverLoader(), new BogusDiscoveryDriverLoader());
		dioptase.start();
			
		// get the bogus temperature/light sensors and instanciate some producers
		Sensor sensor1 = dioptase.getThingProperties().getSensor("temperatureSensor");
		Sensor sensor2 = dioptase.getThingProperties().getSensor("lightSensor");
		Sensor sensor3 = dioptase.getThingProperties().getSensor("cpuSensor");
		Sensor sensor4 = dioptase.getThingProperties().getSensor("memorySensor");
		Producer producer1 = dioptase.createSensorProducer("temperatureProducer1", sensor1, 2000);
		Producer producer2 = dioptase.createSensorProducer("temperatureProducer2", sensor1, 5000);
		Producer producer3 = dioptase.createSensorProducer("lightProducer", sensor2, 2000);
		Producer producer4 = dioptase.createSensorProducer("cpuProducer", sensor3, 5000);
		Producer producer5 = dioptase.createSensorProducer("memoryProducer", sensor4, 5000);
		
		System.out.println("Press enter to close the Dioptase instance");
		System.in.read();
		dioptase.shutdown();
	}
}
