package fr.inria.dioptase.examples;

import java.io.IOException;

import fr.inria.dioptase.Dioptase;
import fr.inria.dioptase.atomic.Processor;
import fr.inria.dioptase.atomic.Producer;
import fr.inria.dioptase.driver.discovery.BogusDiscoveryDriverLoader;
import fr.inria.dioptase.driver.network.JSENetworkDriverLoader;
import fr.inria.dioptase.driver.storage.JSEStorageDriverLoader;
import fr.inria.dioptase.driver.thing.BogusThingDriverLoader;
import fr.inria.dioptase.logic.operator.Average;
import fr.inria.dioptase.logic.operator.Count;
import fr.inria.dioptase.network.web.HttpResponse;
import fr.inria.dioptase.network.web.ServiceInvoker;
import fr.inria.dioptase.service.ThingServices;
import fr.inria.dioptase.stream.Stream;
import fr.inria.dioptase.thing.Sensor;
import fr.inria.dioptase.utils.JSONFormatter;
import fr.inria.dioptase.utils.Logger;
import fr.inria.dioptase.utils.Logger.Level;


/**
 * @author Benjamin Billet
 *
 */
public class MetadataServices
{
	public static void main(String[] args)
	throws IOException
	{
		Logger.setLevel(Level.INFO);
		Dioptase dioptase = Dioptase.getInstance(5000);
		
		dioptase.initDrivers(new JSENetworkDriverLoader(), new BogusThingDriverLoader(), new JSEStorageDriverLoader(), new BogusDiscoveryDriverLoader());
		dioptase.start();
			
		// get the bogus temperature sensor and instanciate a producer
		Sensor sensor = dioptase.getThingProperties().getSensor("temperatureSensor");
		Producer producer = dioptase.createSensorProducer("temperatureProducer", sensor, 1500);
		
		ServiceInvoker invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, ThingServices.SensorsService);
		HttpResponse response = invoker.invoke();
		String sensors = JSONFormatter.format(response.getBody().getContentAsString());
		System.out.println("============ sensors:\n" + sensors);
		
		invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, ThingServices.ActuatorsService);
		response = invoker.invoke();
		String actuators = JSONFormatter.format(response.getBody().getContentAsString());
		System.out.println("============ actuators:\n" + actuators);
		
		invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, ThingServices.OperatorsService);
		response = invoker.invoke();
		String operators = JSONFormatter.format(response.getBody().getContentAsString());
		System.out.println("============ operators:\n" + operators);
		
		invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, ThingServices.InstructionsService);
		response = invoker.invoke();
		String instructions = JSONFormatter.format(response.getBody().getContentAsString());
		System.out.println("============ instructions:\n" + instructions);
		
		invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, ThingServices.PropertiesService);
		response = invoker.invoke();
		String properties = JSONFormatter.format(response.getBody().getContentAsString());
		System.out.println("============ properties:\n" + properties);
		
		// deploy two chained processors
		Processor processor1 = dioptase.createProcessor("processor1", new Average("temperature"), new Stream[] { producer.getStream() });
		Processor processor2 = dioptase.createProcessor("processor2", new Count(), new Stream[] { processor1.getStream() });		
		
		invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, ThingServices.RunningSchemaService);
		response = invoker.invoke();
		String schema = JSONFormatter.format(response.getBody().getContentAsString());
		System.out.println("============ schema:\n" + schema);

		dioptase.shutdown();
	}
}
