package fr.inria.dioptase.examples;

import java.io.IOException;

import fr.inria.dioptase.Dioptase;
import fr.inria.dioptase.atomic.Processor;
import fr.inria.dioptase.atomic.Producer;
import fr.inria.dioptase.driver.discovery.BogusDiscoveryDriverLoader;
import fr.inria.dioptase.driver.network.JSENetworkDriverLoader;
import fr.inria.dioptase.driver.storage.JSEStorageDriverLoader;
import fr.inria.dioptase.driver.thing.BogusThingDriverLoader;
import fr.inria.dioptase.logic.interpreter.InterpretedProgram;
import fr.inria.dioptase.logic.interpreter.ProgramParser;
import fr.inria.dioptase.stream.DataType;
import fr.inria.dioptase.stream.Stream;
import fr.inria.dioptase.stream.StreamCallback;
import fr.inria.dioptase.stream.StreamItem;
import fr.inria.dioptase.stream.StreamSchema;
import fr.inria.dioptase.thing.Sensor;
import fr.inria.dioptase.utils.Logger;
import fr.inria.dioptase.utils.Logger.Level;


/**
 * @author Benjamin Billet
 *
 */
public class SimpleProcessor
{
	public static void main(String[] args)
	throws IOException
	{
		Logger.setLevel(Level.INFO);
		Dioptase dioptase = Dioptase.getInstance(5000);
		
		dioptase.initDrivers(new JSENetworkDriverLoader(), new BogusThingDriverLoader(), new JSEStorageDriverLoader(), new BogusDiscoveryDriverLoader());
		dioptase.start();
		
		// get the bogus temperature sensor and instanciate a producer
		Sensor sensor = dioptase.getThingProperties().getSensor("temperatureSensor");
		Producer producer = dioptase.createSensorProducer("temperatureProducer", sensor, 2000);
		
		// compile the AVERAGE task
		InterpretedProgram program = new ProgramParser().parseProgram(getAVERAGEDescription(producer.getStream(), "temperature"), new StreamSchema("avg", new DataType("temperature", "celcius", "real")));
		// and deploy it into a processor, connected to the producer
		Processor processor = dioptase.createProcessor("avgProcessor", program, new Stream[] { producer.getStream() });
		
		System.out.println("============ Producer schema:");
		System.out.println(producer.getId() + " " + producer.getStream().getSchema());
		System.out.println("============ Processor schema:");
		System.out.println(processor.getId() + " " + processor.getStream().getSchema());
		
		System.out.println("============ Outputs (press enter to exit):");
		
		// read the output of the producer
		producer.getDispatcher().addCallback(new StreamCallback()
		{				
			@Override
			public void newItemReceived(Stream stream, StreamItem item)
			{
				System.out.println(item.getTimestamp() + ": " + stream.getName() + " produces " + item.getTuple());
			}
		});
		
		// read the output of the processor
		processor.getDispatcher().addCallback(new StreamCallback()
		{				
			@Override
			public void newItemReceived(Stream stream, StreamItem item)
			{
				System.out.println(item.getTimestamp() + ": " + stream.getName() + " produces " + item.getTuple());
			}
		});
		
		System.in.read();
		dioptase.shutdown();
	}
	
	public static String getAVERAGEDescription(Stream input, String attribute)
	{
		return
				"init:" +
				"	(defglobal \"sum\" 0)" +
				"	(defglobal \"count\" 0)" +
				"work:" +
				"	(def \"diff\" (getNewItems \"" + input.getName() + "\"))" +
				"	(if (> (size diff) 0)" +
				"		((set \"count\" (+ count (size diff)))" +
				"		(foreach \"i\" diff" +
				"			(set \"sum\" (+ sum (itemvalue i \"" + attribute + "\")))" +
				"		)" +
				"		(write (item (now) \"avg\" (/ sum count))))\r\n" +
				"	)\r\n";
	}
}
