package fr.inria.dioptase.examples;

import java.io.IOException;

import fr.inria.dioptase.Dioptase;
import fr.inria.dioptase.atomic.Producer;
import fr.inria.dioptase.driver.discovery.BogusDiscoveryDriverLoader;
import fr.inria.dioptase.driver.network.JSENetworkDriverLoader;
import fr.inria.dioptase.driver.storage.JSEStorageDriverLoader;
import fr.inria.dioptase.driver.thing.BogusThingDriverLoader;
import fr.inria.dioptase.stream.Stream;
import fr.inria.dioptase.stream.StreamCallback;
import fr.inria.dioptase.stream.StreamItem;
import fr.inria.dioptase.thing.Sensor;
import fr.inria.dioptase.utils.Logger;
import fr.inria.dioptase.utils.Logger.Level;


/**
 * @author Benjamin Billet
 *
 */
public class SimpleProducer
{
	public static void main(String[] args)
	throws IOException
	{
		Logger.setLevel(Level.INFO);
		Dioptase dioptase = Dioptase.getInstance(5000);
		
		dioptase.initDrivers(new JSENetworkDriverLoader(), new BogusThingDriverLoader(), new JSEStorageDriverLoader(), new BogusDiscoveryDriverLoader());
		dioptase.start();
			
		// get the bogus temperature sensor and instanciate a producer
		Sensor sensor = dioptase.getThingProperties().getSensor("temperatureSensor");
		Producer producer = dioptase.createSensorProducer("myTemperatureProducer", sensor, 1500);
		
		System.out.println("============ Producer schema:");
		System.out.println(producer.getStream().getSchema());
		System.out.println("============ Producer output (press enter to exit):");
		
		// read the output of the producer
		producer.getDispatcher().addCallback(new StreamCallback()
		{				
			@Override
			public void newItemReceived(Stream stream, StreamItem item)
			{
				System.out.println(item.getTimestamp() + ": " + stream.getName() + " produces " + item.getTuple());
			}
		});
		
		System.in.read();
		dioptase.shutdown();
	}
}
