package fr.inria.dioptase.examples;

import java.io.IOException;
import java.util.Vector;

import fr.inria.dioptase.Dioptase;
import fr.inria.dioptase.atomic.Producer;
import fr.inria.dioptase.atomic.Storage;
import fr.inria.dioptase.driver.discovery.BogusDiscoveryDriverLoader;
import fr.inria.dioptase.driver.network.JSENetworkDriverLoader;
import fr.inria.dioptase.driver.storage.JSEStorageDriverLoader;
import fr.inria.dioptase.driver.thing.BogusThingDriverLoader;
import fr.inria.dioptase.network.web.ServiceInvoker;
import fr.inria.dioptase.service.StorageServices;
import fr.inria.dioptase.stream.Stream;
import fr.inria.dioptase.stream.StreamCallback;
import fr.inria.dioptase.stream.StreamItem;
import fr.inria.dioptase.thing.Sensor;
import fr.inria.dioptase.utils.Logger;
import fr.inria.dioptase.utils.Logger.Level;
import fr.inria.dioptase.utils.Time;


/**
 * @author Benjamin Billet
 *
 */
public class SimpleStorage
{
	public static void main(String[] args)
	throws IOException, InterruptedException
	{
		Logger.setLevel(Level.INFO);
		Dioptase dioptase = Dioptase.getInstance(5000);
		
		dioptase.initDrivers(new JSENetworkDriverLoader(), new BogusThingDriverLoader(), new JSEStorageDriverLoader(), new BogusDiscoveryDriverLoader());
		dioptase.start();
			
		// get the bogus temperature sensor and instanciate a producer
		Sensor sensor = dioptase.getThingProperties().getSensor("temperatureSensor");
		Producer producer = dioptase.createSensorProducer("temperatureProducer", sensor, 300);
		
		ServiceInvoker invoker = new ServiceInvoker(dioptase.getContext(), "127.0.0.1", 5000, StorageServices.DeployStorageService);
		invoker.setServiceParameter("type", "Memory");
		invoker.setServiceParameter("id", "temperatureStorage");
		invoker.setServiceParameter("input", "temperatureProducer");
		
		invoker.invoke();
		Storage storage = dioptase.findStorage("temperatureStorage");
		
		System.out.println("Please wait five seconds while the temperature producer generates data.");
		
		// read the output of the producer
		producer.getDispatcher().addCallback(new StreamCallback()
		{				
			@Override
			public void newItemReceived(Stream stream, StreamItem item)
			{
				System.out.println(item.getTimestamp() + ": " + stream.getName() + " produces " + item.getTuple());
			}
		});

		
		Thread.sleep(5000);
		dioptase.deleteProducer("temperatureProducer");
		
		System.out.println("============ Storage content:");
		displayStorageData(storage.readAll());
		
		System.out.println("============ 5 last stored items:");
		displayStorageData(storage.readSome(5));
		
		System.out.println("============ items that were create within the 2 last seconds:");
		displayStorageData(storage.readSince(Time.now() - 2000));
		
		dioptase.shutdown();
	}
	
	private static void displayStorageData(Vector items)
	{
		for(Object i : items)
		{
			System.out.println(i);
		}
	}
}

